@@ -15,21 +15,20 @@ namespace builtins
   {
 
     template <typename T, typename... Types>
-    types::set<T> difference(types::set<T> const &set, Types const &... others)
+    types::set<T> difference(types::set<T> const &set, Types const &...others)
     {
       return set.difference(others...);
     }
 
     template <typename T, typename... Types>
-    types::set<T> difference(types::set<T> &&set, Types const &... others)
+    types::set<T> difference(types::set<T> &&set, Types const &...others)
     {
       set.difference_update(others...);
       return set;
     }
 
     template <typename... Types>
-    types::empty_set difference(types::empty_set const &set,
-                                Types const &... others)
+    types::empty_set difference(types::empty_set const &set, Types const &...others)
     {
       return types::empty_set();
     }
@@ -46,11 +45,11 @@ namespace builtins
       return set;
     }
 
-    types::empty_set difference(types::empty_set const &set)
+    inline types::empty_set difference(types::empty_set const &set)
     {
       return types::empty_set();
     }
-  }
-}
+  } // namespace set
+} // namespace builtins
 PYTHONIC_NS_END
 #endif
