@@ -9,11 +9,10 @@ namespace numpy
 {
 
   template <class ArraySequence>
-  auto hstack(ArraySequence &&seq)
-      -> decltype(concatenate(std::forward<ArraySequence>(seq), 1));
+  auto hstack(ArraySequence &&seq) -> decltype(concatenate(std::forward<ArraySequence>(seq), 1));
 
   DEFINE_FUNCTOR(pythonic::numpy, hstack);
-}
+} // namespace numpy
 PYTHONIC_NS_END
 
 #endif
