@@ -15,13 +15,13 @@ namespace numpy
     types::ndarray<double, pS> exponential(double scale, pS const &shape);
 
     auto exponential(double scale, long size)
-        -> decltype(exponential(scale, types::array<long, 1>{{size}}));
+        -> decltype(exponential(scale, types::array_tuple<long, 1>{{size}}));
 
     double exponential(double scale = 1.0, types::none_type size = {});
 
     DEFINE_FUNCTOR(pythonic::numpy::random, exponential);
-  }
-}
+  } // namespace random
+} // namespace numpy
 PYTHONIC_NS_END
 
 #endif
