@@ -22,19 +22,16 @@ namespace operator_
 {
 
   template <class A, class B>
-  auto OPERATOR_NAME(bool, A &&a, B &&b,
-                     ...) -> decltype(std::forward<A>(a)
-                                          OPERATOR_SYMBOL std::forward<B>(b));
+  auto OPERATOR_NAME(bool, A &&a, B &&b, ...)
+      -> decltype(std::forward<A>(a) OPERATOR_SYMBOL std::forward<B>(b));
 
   template <class A, class B>
   auto OPERATOR_NAME(bool, A &&a, B &&b, std::nullptr_t)
       -> decltype(std::forward<A>(a) OPERATOR_ISYMBOL std::forward<B>(b));
 
   template <class A, class B>
-  auto OPERATOR_NAME(A &&a,
-                     B &&b) -> decltype(OPERATOR_NAME(true, std::forward<A>(a),
-                                                      std::forward<B>(b),
-                                                      nullptr))
+  auto OPERATOR_NAME(A &&a, B &&b)
+      -> decltype(OPERATOR_NAME(true, std::forward<A>(a), std::forward<B>(b), nullptr))
   {
     return OPERATOR_NAME(true, std::forward<A>(a), std::forward<B>(b), nullptr);
   }
