@@ -13,8 +13,7 @@ PYTHONIC_NS_BEGIN
 namespace numpy
 {
   template <class T, class pS, class E, class F>
-  types::none_type putmask(types::ndarray<T, pS> &expr, E const &mask,
-                           F const &values)
+  types::none_type putmask(types::ndarray<T, pS> &expr, E const &mask, F const &values)
   {
     auto amask = asarray(mask);
     auto avalues = asarray(values);
