@@ -3,8 +3,8 @@
 
 #include "pythonic/include/numpy/zeros_like.hpp"
 
-#include "pythonic/utils/functor.hpp"
 #include "pythonic/numpy/zeros.hpp"
+#include "pythonic/utils/functor.hpp"
 
 PYTHONIC_NS_BEGIN
 
@@ -12,20 +12,18 @@ namespace numpy
 {
 
   template <class E, class dtype>
-  auto zeros_like(E const &expr, dtype d)
-      -> decltype(zeros(sutils::getshape(expr), d))
+  auto zeros_like(E const &expr, dtype d) -> decltype(zeros(sutils::getshape(expr), d))
   {
     return zeros(sutils::getshape(expr), d);
   }
 
   template <class E>
   auto zeros_like(E const &expr, types::none_type)
-      -> decltype(zeros(sutils::getshape(expr),
-                        types::dtype_t<typename E::dtype>()))
+      -> decltype(zeros(sutils::getshape(expr), types::dtype_t<typename E::dtype>()))
   {
     return zeros(sutils::getshape(expr), types::dtype_t<typename E::dtype>());
   }
-}
+} // namespace numpy
 PYTHONIC_NS_END
 
 #endif
