@@ -3,12 +3,12 @@
 
 #include "pythonic/include/scipy/special/hankel2.hpp"
 
-#include "pythonic/types/ndarray.hpp"
 #include "pythonic/types/complex.hpp"
+#include "pythonic/types/ndarray.hpp"
 #include "pythonic/utils/functor.hpp"
 #include "pythonic/utils/numpy_traits.hpp"
 
-#define BOOST_MATH_THREAD_LOCAL thread_local
+#include "pythonic/utils/boost_local_config.hpp"
 #include <boost/math/special_functions/hankel.hpp>
 
 PYTHONIC_NS_BEGIN
@@ -24,13 +24,13 @@ namespace scipy
       {
         return boost::math::cyl_hankel_2(x, y);
       }
-    }
+    } // namespace details
 
 #define NUMPY_NARY_FUNC_NAME hankel2
 #define NUMPY_NARY_FUNC_SYM details::hankel2
 #include "pythonic/types/numpy_nary_expr.hpp"
-  }
-}
+  } // namespace special
+} // namespace scipy
 PYTHONIC_NS_END
 
 #endif
