@@ -28,7 +28,7 @@ from pythran import compile_pythrancode, spec_parser, load_specfile, frontend
 from pythran.backend import Python
 from pythran.middlend import refine
 from pythran.passmanager import PassManager
-from pythran.toolchain import _parse_optimization
+from pythran.toolchain import _parse_optimization, generate_cxx
 from pythran.spec import Spec
 
 logger = logging.getLogger("pythran")
@@ -94,12 +94,19 @@ class TestEnv(unittest.TestCase):
             self.assertIsInstance(res, (float, float64))
         elif isinstance(ref, (complex, complex128)):
             self.assertIsInstance(res, (complex, complex128))
+        elif isinstance(ref, complex256):
+            self.assertIsInstance(res, complex256)
         elif isinstance(ref, (bool, bool_)):
             self.assertIsInstance(res, (bool, bool_))
         elif hasattr(ref, 'dtype'):
             if hasattr(res, 'dtype'):
+                def issigned(ty):
+                    try:
+                        return ty(-1) == -1
+                    except OverflowError:
+                        return False
                 self.assertEqual(ref.dtype.itemsize, res.dtype.itemsize)
-                self.assertEqual(ref.dtype.type(-1), res.dtype.type(-1))
+                self.assertEqual(issigned(ref.dtype.type), issigned(res.dtype.type))
             else:
                 self.assertIsInstance(res, int)
         else:
@@ -227,7 +234,7 @@ class TestEnv(unittest.TestCase):
         Raises:
            AssertionError by 'unittest' if return value differ.
            SyntaxError if code is not python valid.
-           pythran.CompileError if generated code can't be compiled.
+           pythran.errors.PythranCompileError if generated code can't be compiled.
            ...possibly others...
         """
         # Extract special keys from interface.
@@ -284,7 +291,7 @@ class TestEnv(unittest.TestCase):
         Raises:
            AssertionError by 'unittest' if return value differ.
            SyntaxError if code is not python valid.
-           pythran.CompileError if generated code can't be compiled.
+           pythran.errors.PythranCompileError if generated code can't be compiled.
            ...possibly others...
         """
         # Extract special keys from interface.
@@ -299,6 +306,12 @@ class TestEnv(unittest.TestCase):
 
         code = dedent(code)
 
+        cxx0, _ = generate_cxx(modname, code, interface)
+        cxx1, _ = generate_cxx(modname, code, interface)
+        self.maxDiff, maxDiff = None, self.maxDiff
+        self.assertEqual(str(cxx0), str(cxx1))
+        self.maxDiff = maxDiff
+
         cxx_compiled = compile_pythrancode(
             modname, code, interface, extra_compile_args=self.PYTHRAN_CXX_FLAGS)
 
