@@ -5,6 +5,7 @@ from unittest import skip, skipIf
 import numpy
 
 from pythran.typing import *
+from packaging.version import Version
 
 class TestAdvanced(TestEnv):
 
@@ -85,15 +86,6 @@ def value_fill_max(x, dic):
     def test_parallel_enumerate(self):
         self.run_test('def parallel_enumerate(l):\n k = [0]*(len(l) + 1)\n "omp parallel for"\n for i,j in enumerate(l):\n  k[i+1] = j\n return k', list(range(1000)), parallel_enumerate=[List[int]])
 
-    def test_ultra_nested_functions(self):
-        code = '''
-def ultra_nested_function(n):
-	def foo(y):
-		def bar(t): return t
-		return bar(y)
-	return foo(n)'''
-        self.run_test(code, 42, ultra_nested_function=[int])
-
     def test_generator_sum(self):
         code = '''
 def generator_sum(l0,l1):
@@ -197,6 +189,28 @@ def combiner_on_empty_list():
         with self.assertRaises(SyntaxError):
             self.run_test(code, default_arg6=[])
 
+    def test_default_arg7(self):
+        code = '''
+import numpy as np
+
+def A(I0, I1=0):
+    AA = I1
+    A2 = 1000
+    if AA:
+        A2 //= 2
+    self = tuple([[AA],[A2]])
+    return self
+
+def B(sRate):
+    BB = A(np.empty((sRate, 2)))
+    self = tuple([[BB]])
+    return self
+
+def default_arg7(n):
+    BBB = B(n)
+    return BBB'''
+        self.run_test(code, 48000, default_arg7=[int])
+
     @skip("lists as zeros parameter are not supported")
     def test_list_as_zeros_parameter(self):
         self.run_test('def list_as_zeros_parameter(n): from numpy import zeros ; return zeros([n,n])', 3, list_as_zeros_parameter=[int])
@@ -287,6 +301,15 @@ def combiner_on_empty_list():
             numpy.array([[0., 2.], [1., 3.]]),
             matmul_operator=[NDArray[float, :,:], NDArray[float, :,:]])
 
+    @skipIf(Version(numpy.__version__) <= Version('1.26'), "Not supported upstream")
+    def test_imatmul_operator(self):
+        code = 'def imatmul_operator(x, y): x @= y; return x'
+        self.run_test(
+            code,
+            numpy.array([[1., 1.], [2., 2.]]),
+            numpy.array([[0., 2.], [1., 3.]]),
+            imatmul_operator=[NDArray[float, :,:], NDArray[float, :,:]])
+
     def test_generator_handler_name(self):
         code = '''
             def foo(x):
