@@ -553,17 +553,37 @@ def np_rosen_der(x):
         finally:
             os.remove(temp_name)
 
+    @unittest.skipIf(np_version > version.Version("2.2"), reason="np.fromstring no longer supports binary mode")
     def test_fromstring0(self):
         self.run_test("def np_fromstring0(a): from numpy import fromstring, uint8 ; return fromstring(a, uint8)", '\x01\x02', np_fromstring0=[str])
 
+    @unittest.skipIf(np_version > version.Version("2.2"), reason="np.fromstring no longer supports binary mode")
     def test_fromstring1(self):
         self.run_test("def np_fromstring1(a): from numpy import fromstring, uint8 ; a = '\x01\x02\x03\x04' ; return fromstring(a, uint8,3)", '\x01\x02\x03\x04', np_fromstring1=[str])
 
     def test_fromstring2(self):
-        self.run_test("def np_fromstring2(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32,-1, ' ')", '1 2 3 4', np_fromstring2=[str])
+        self.run_test("def np_fromstring2(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32,-1, sep=' ')", '1 20 3 40', np_fromstring2=[str])
 
     def test_fromstring3(self):
-        self.run_test("def np_fromstring3(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32,2, ',')", '1,2, 3, 4', np_fromstring3=[str])
+        self.run_test("def np_fromstring3(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32,2, sep=',')", '1,2, 3, 4', np_fromstring3=[str])
+
+    def test_fromstring4(self):
+        self.run_test("def np_fromstring4(a): from numpy import fromstring ; return fromstring(a, sep=',')", '1.1, 2.2, 3.3, 4.4', np_fromstring4=[str])
+
+    def test_fromstring5(self):
+        self.run_test("def np_fromstring5(a): from numpy import fromstring, uint32 ; return fromstring(a, uint32, sep=',')", '1, 2, 3, 4', np_fromstring5=[str])
+
+    def test_frombuffer0(self):
+        self.run_test("def np_frombuffer0(a): from numpy import frombuffer, uint8 ; return frombuffer(b'\x01\x02' * a, uint8)", 1, np_frombuffer0=[int])
+
+    def test_frombuffer1(self):
+        self.run_test("def np_frombuffer1(a): from numpy import frombuffer, uint8 ; return frombuffer(b'\x01\x02\x03\x04' * a, uint8, 3)", 1, np_frombuffer1=[int])
+
+    def test_frombuffer2(self):
+        self.run_test("def np_frombuffer2(a): from numpy import frombuffer, uint16 ; return frombuffer(a * b'\x01\x02', uint16)", 1, np_frombuffer2=[int])
+
+    def test_frombuffer3(self):
+        self.run_test("def np_frombuffer3(a): from numpy import frombuffer, int8 ; return frombuffer(a * b'\x01\x02\x03\x04', int8, 3, 1)", 1, np_frombuffer3=[int])
 
     def test_outer0(self):
         self.run_test("def np_outer0(x): from numpy import outer ; return outer(x, x+2)", numpy.arange(6).reshape(2,3), np_outer0=[NDArray[int,:,:]])
@@ -819,6 +839,12 @@ def np_ravel3(v):
     def test_sort_complex1(self):
         self.run_test("def np_sort_complex1(a): from numpy import sort_complex ; return sort_complex(a)", numpy.array([1 + 2j, 2 - 1j, 3 - 2j, 3 - 3j, 3 + 5j]), np_sort_complex1=[NDArray[complex,:]])
 
+    def test_sort_nan0(self):
+        self.run_test("def np_sort_nan0(x): from numpy import sort; return sort(x)", numpy.array([1., numpy.nan, 0., numpy.nan]), np_sort_nan0=[NDArray[float,:]])
+
+    def test_sort_nan1(self):
+        self.run_test("def np_sort_nan1(x): from numpy import sort; return sort(x)", numpy.array([1., numpy.nan, 0., numpy.nan], dtype=numpy.float32), np_sort_nan1=[NDArray[numpy.float32,:]])
+
     def test_split0(self):
         self.run_test("def np_split0(a): from numpy import split,array2string ; return list(map(array2string,split(a, 3)))", numpy.arange(12), np_split0=[NDArray[int,:]])
 
