@@ -206,6 +206,12 @@ class TestNumpyFunc2(TestEnv):
                   numpy.arange(10,dtype=float),
                   np_convolve_2=[NDArray[float,:],NDArray[float,:]])
 
+    def test_convolve_2b(self):
+        self.run_test("def np_convolve_2b(a,b):\n from numpy import convolve\n return convolve(a,b)",
+                  numpy.arange(12,dtype=numpy.float128),
+                  numpy.arange(10,dtype=numpy.float128),
+                  np_convolve_2b=[NDArray[numpy.float128,:],NDArray[numpy.float128,:]])
+
     def test_convolve_3(self):
         self.run_test("def np_convolve_3(a,b):\n from numpy import convolve\n return convolve(a,b,'valid')",
                   numpy.arange(12,dtype=float),
@@ -615,7 +621,7 @@ def test_copy0(x):
         self.run_test("def np_around5(x): from numpy import around ; return around(x,-1)", [1,2,3,11], np_around5=[List[int]])
 
     def test_argsort0(self):
-        self.run_test("def np_argsort0(x): from numpy import argsort ; return argsort(x)", numpy.array([3, 1, 2]), np_argsort0=[NDArray[int,:]])
+        self.run_test("import numpy\ndef np_argsort0(x: numpy.ndarray) -> numpy.ndarray: from numpy import argsort ; return argsort(x)", numpy.array([3, 1, 2]), np_argsort0=[NDArray[int,:]])
 
     def test_argsort1(self):
         self.run_test("def np_argsort1(x): return x.argsort()", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort1=[NDArray[int,:,:]])
@@ -737,6 +743,9 @@ def test_copy0(x):
     def test_array_iter(self):
         self.run_test("def np_array_iter():\n from numpy import array\n return array(list(reversed(range(0, 300)))) + 3", np_array_iter=[])
 
+    def test_array_tuple(self):
+        self.run_test("def np_array_tuple(x):\n from numpy import array\n return array((1, 2, x))", 3.5, np_array_tuple=[float])
+
     def test_array_typed(self):
         self.run_test("def np_array_typed(a):\n from numpy import array, int64\n return array(a, int64)", [1.,2.,3.], np_array_typed=[List[float]])
 
@@ -887,6 +896,24 @@ def test_copy0(x):
     def test_linspace4(self):
         self.run_test("def np_linspace4_(a):\n from numpy import linspace\n return linspace(1,a,dtype=int)", 4, np_linspace4_=[int])
 
+    def test_linspace5(self):
+        self.run_test("def np_linspace5_(a):\n from numpy import linspace\n return linspace(0,a,1)", 1, np_linspace5_=[int])
+
+    def test_linspace6(self):
+        self.run_test("def np_linspace6_(a):\n from numpy import linspace\n return linspace(0,a,1)", 1., np_linspace6_=[float])
+
+    def test_linspace7(self):
+        self.run_test("def np_linspace7_(a):\n from numpy import linspace\n return linspace(1,a,1)", 1, np_linspace7_=[int])
+
+    def test_linspace8(self):
+        self.run_test("def np_linspace8_(a):\n from numpy import linspace\n return linspace(0,a,0)", 0, np_linspace8_=[int])
+
+    def test_linspace9(self):
+        self.run_test("def np_linspace9_(a):\n from numpy import linspace\n return linspace(0,a,0)", 1, np_linspace9_=[int])
+
+    def test_linspace10(self):
+        self.run_test("def np_linspace10_(a):\n from numpy import linspace\n return linspace(0,a,1)", -.5, np_linspace10_=[float])
+
     def test_sin(self):
         self.run_test("def np_sin_(a):\n from numpy import sin\n return sin(a)", numpy.linspace(0,6), np_sin_=[NDArray[float,:]])
 
