@@ -44,7 +44,7 @@ class TestNumpyLinalg(TestEnv):
                       b = a.reshape((3, 3))
                       c = np.array([[ 1, 2, 3], [-1, 1, x]])
                       return (LA.norm(a), LA.norm(b),
-                              LA.norm(a, np.Inf), #LA.norm(b, np.inf),
+                              LA.norm(a, np.inf), #LA.norm(b, np.inf),
                               LA.norm(a, -np.inf), #LA.norm(b, -np.inf),
                               LA.norm(a, 1), #LA.norm(b, 1),
                               LA.norm(a, -1), #LA.norm(b, -1),
