@@ -12,6 +12,20 @@ class TestOptimization(TestEnv):
         code = "def constant_fold_nan(a): from numpy import nan; a[0] = nan; return a"
         self.run_test(code, [1., 2.], constant_fold_nan=[List[float]])
 
+    def test_constant_fold_restrict_assign(self):
+        code = """
+import numpy as np
+def make_signal():
+    y = np.ones((44100,2))
+    y[:, 1:] = 0. #<--- the issue.
+    return y
+
+def constant_fold_restrict_assign():
+    x = make_signal()
+    return x
+    """
+        self.run_test(code, constant_fold_restrict_assign=[])
+
     def test_constant_fold_subscript(self):
         code = '''
 def aux(n):
@@ -288,7 +302,7 @@ def foo():
     'omp parallel'
     if 1:
         builtins.print(a)
-    return builtins.None"""
+    return None"""
         self.check_ast(init, ref, ["pythran.optimizations.ForwardSubstitution"])
 
     def test_omp_forwarding2(self):
@@ -305,7 +319,7 @@ def foo():
     if 1:
         pass
         builtins.print(2)
-    return builtins.None"""
+    return None"""
         self.check_ast(init, ref, ["pythran.optimizations.ForwardSubstitution"])
 
     def test_omp_forwarding3(self):
@@ -322,7 +336,7 @@ def foo():
     if 1:
         a = 2
     builtins.print(a)
-    return builtins.None"""
+    return None"""
         self.check_ast(init, ref, ["pythran.optimizations.ForwardSubstitution"])
 
     def test_forwarding0(self):
@@ -564,6 +578,10 @@ class TestConstantUnfolding(TestEnv):
     def test_constant_folding_set_literals(self):
         self.run_test("def constant_folding_set_literals(): return {1,2,3,3}", constant_folding_set_literals=[])
 
+    def test_constant_folding_comparators(self):
+        self.run_test("def constant_folding_comparators(): return 1 > 1, 1>=1, 1 < 1, 1 <= 1, 1 == 1, 1 != 1, 1 is 2, (1 < 2 < 3), 1 in [1, 2] in [(1,2)], 1 not in [1, 2]",
+                      constant_folding_comparators=[])
+
     def test_constant_folding_builtins(self):
         self.run_test("def constant_folding_builtins(): return list(map(len,zip(range(2), range(2))))", constant_folding_builtins=[])
 
@@ -793,3 +811,17 @@ def ftc (b, c, y, t, m):
                 else:
                     return 0'''
         self.run_test(code, [], insert_none0=[List[int]])
+
+    def test_inline_in_while_test(self):
+        code = '''
+def is_even(n: int) -> int:
+    return (n & 1) == 0
+
+def inline_in_while_test(n: int) -> int:
+    result = 0
+    while is_even(n):
+        n >>= 1
+        result += 1
+    return result
+        '''
+        self.run_test(code, 7, inline_in_while_test=[int])
