@@ -5,7 +5,7 @@ import pythran
 from textwrap import dedent
 
 from pythran.typing import List, Dict, NDArray, Tuple
-import pythran.types.tog
+from pythran.errors import PythranTypeError
 
 
 class TestTyping(TestEnv):
@@ -126,6 +126,29 @@ def dict_of_set():
         '''
         self.run_test(code, 0, typing_aliasing_and_variable_subscript=[int])
 
+    def test_typing_aliasing_and_variable_subscript_through_call(self):
+        code = '''
+        def typing_aliasing_and_variable_subscript_through_call(i):
+            a: list[type(i)] =[]
+            b=[a]
+            (lambda x: b[0].append(x))(i)
+            return a, b
+        '''
+        self.run_test(code, 0,
+                      typing_aliasing_and_variable_subscript_through_call=[int])
+
+    def test_typing_aliasing_and_variable_subscript_through_call_set(self):
+        code = '''
+        def typing_aliasing_and_variable_subscript_through_call_list_list(i):
+            a: list[type(i)] = []
+            b=[(a,)]
+            #(lambda x: list(b)[0][0].append(x))(i)
+            list(b)[0][0].append(i)
+            return a, b
+        '''
+        self.run_test(code, 0,
+                      typing_aliasing_and_variable_subscript_through_call_list_list=[int])
+
     def test_typing_aliasing_and_variable_subscript_combiner(self):
         code = '''
         def typing_aliasing_and_variable_subscript_combiner(i):
@@ -423,7 +446,7 @@ def recursive_interprocedural_typing2(c):
             slice_assign=[int])
 
     def verify_type_error(self, code):
-        with self.assertRaises(pythran.types.tog.PythranTypeError):
+        with self.assertRaises(PythranTypeError):
             _, eh = pythran.generate_cxx("dumbo", dedent(code))
             eh()
 
@@ -750,3 +773,87 @@ def numpy_array_in_tuple1(N = 10):
     CL = CL___init__(N)
     return CL_test(CL)'''
         return self.run_test(code, 6, numpy_array_in_tuple1=[int])
+
+    def test_complex_slice_combination(self):
+        code = '''
+import numpy
+def complex_slice_combination(x):
+    arr = numpy.empty(shape = (x, 1))
+    a1 = 1
+    def save_step():
+        arr[0][:] = [
+            a1,
+        ]
+    save_step()
+    return arr'''
+        return self.run_test(code, 1, complex_slice_combination=[int])
+
+    def test_augassign_update_dict_interprocedural(self):
+        code = '''
+def augassign_update_dict_interprocedural(cache):
+    cache_slice = cache[1,2]
+    aaa = [cache_slice[3], cache_slice[2]]
+    def do_thing ():
+        aaa[0] -= 1
+    do_thing ()'''
+        return self.run_test(code, np.arange(2*3*4).reshape((2,3,4)),
+        augassign_update_dict_interprocedural=[NDArray[int,:,:,:]])
+
+    def test_augassign_both_branches(self):
+        code = '''
+def augassign_both_branches(
+    x, y, z
+):
+    g = [1,2,3]
+    bools = [True, True, True]
+    ret = 0.
+    if bools[y]:
+        ret += g[y]
+        ret += 123
+    else:
+        ret += g[y]
+        ret += 234
+    return ret'''
+        return self.run_test(code, np.arange(4), 1, 2,
+                             augassign_both_branches=[NDArray[int,:], int, int])
+
+    def test_assign_interprocedural_subscript(self):
+        code = '''
+import numpy as np
+
+def assign_interprocedural_subscript(N):
+    self = ([0], [np.zeros(N)])
+    B(self, N)
+    return self
+
+def B(self, N):
+    mB = int(N)
+    self[0][0] = mB
+    self[1][0] = np.zeros(mB)
+    C(self)
+
+def C(self):
+    self[1][0][0:1] = self[1][0][0:1]*2'''
+        return self.run_test(code, 10, assign_interprocedural_subscript=[int])
+
+    def test_expr_cast(self):
+        code = '''
+def test(x):
+    import numpy as np
+    N = x.shape[0]
+    out = np.empty_like(x, np.float32)
+    for i in range(N):
+        if x[i] > 0:
+            out[i] = x[i]
+        else:
+            out[i] = 0
+    return out
+
+# pythran export f(float32[:, :]):
+def test_expr_cast(x):
+    import numpy as np
+    out = np.empty_like(x, np.float32)
+    out[0] = test(np.abs(x[0]))
+    return out'''
+        self.run_test(code, np.ones((1,1), dtype=np.float32),
+                      test_expr_cast=[NDArray[np.float32, :, :]])
